@@ -1,5 +1,5 @@
 // Symphonia
-// Copyright (c) 2019 The Project Symphonia Developers.
+// Copyright (c) 2019-2021 The Project Symphonia Developers.
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -113,7 +113,7 @@ pub struct StreamInfo {
 
 impl StreamInfo {
 
-    pub fn read<B : ByteStream>(reader: &mut B)  -> Result<StreamInfo> {
+    pub fn read<B : ReadBytes>(reader: &mut B)  -> Result<StreamInfo> {
         let mut info = StreamInfo {
             block_len_min: 0,
             block_len_max: 0,
@@ -132,12 +132,12 @@ impl StreamInfo {
 
         // Validate the block length bounds are in the range [16, 65535] samples.
         if info.block_len_min < 16 || info.block_len_max < 16{
-            return decode_error("minimum block length is 16 samples");
+            return decode_error("flac: minimum block length is 16 samples");
         }
 
         // Validate the maximum block size is greater than or equal to the minimum block size.
         if info.block_len_max < info.block_len_min {
-            return decode_error("maximum block length is less than the minimum block length");
+            return decode_error("flac: maximum block length is less than the minimum block length");
         }
 
         // Read the frame byte length bounds.
@@ -151,37 +151,37 @@ impl StreamInfo {
             && info.frame_byte_len_max > 0
             && info.frame_byte_len_max < info.frame_byte_len_min
         {
-            return decode_error("maximum frame length is less than the minimum frame length");
+            return decode_error("flac: maximum frame length is less than the minimum frame length");
         }
 
-        let mut br = BitReaderLtr::new();
+        let mut br = BitStreamLtr::new(reader);
 
         // Read sample rate, valid rates are [1, 655350] Hz.
-        info.sample_rate = br.read_bits_leq32(reader, 20)?;
+        info.sample_rate = br.read_bits_leq32(20)?;
 
         if info.sample_rate < 1 || info.sample_rate > 655_350 {
-            return decode_error("stream sample rate out of bounds");
+            return decode_error("flac: stream sample rate out of bounds");
         }
 
         // Read number of channels minus 1. Valid number of channels are 1-8.
-        let channels_enc = br.read_bits_leq32(reader, 3)? + 1;
+        let channels_enc = br.read_bits_leq32(3)? + 1;
 
         if channels_enc < 1 || channels_enc > 8 {
-            return decode_error("stream channels are out of bounds");
+            return decode_error("flac: stream channels are out of bounds");
         }
 
         info.channels = flac_channels_to_channels(channels_enc);
 
         // Read bits per sample minus 1. Valid number of bits per sample are 4-32.
-        info.bits_per_sample = br.read_bits_leq32(reader, 5)? + 1;
+        info.bits_per_sample = br.read_bits_leq32(5)? + 1;
 
         if info.bits_per_sample < 4 || info.bits_per_sample > 32 {
-            return decode_error("stream bits per sample are out of bounds")
+            return decode_error("flac: stream bits per sample are out of bounds")
         }
 
         // Read the total number of samples. All values are valid. A value of 0 indiciates a stream
         // of unknown length.
-        info.n_samples = match br.read_bits_leq64(reader, 36)? {
+        info.n_samples = match br.read_bits_leq64(36)? {
             0 => None,
             samples => Some(samples)
         };
@@ -193,14 +193,14 @@ impl StreamInfo {
     }
 }
 
-pub fn read_comment_block<B : ByteStream>(
+pub fn read_comment_block<B : ReadBytes>(
     reader: &mut B,
     metadata: &mut MetadataBuilder,
 ) -> Result<()> {
     vorbis::read_comment_no_framing(reader, metadata)
 }
 
-pub fn read_seek_table_block<B : ByteStream>(
+pub fn read_seek_table_block<B : ReadBytes>(
     reader: &mut B,
     block_length: u32,
     table: &mut SeekIndex
@@ -242,14 +242,14 @@ fn printable_ascii_to_string(bytes: &[u8]) -> Option<String> {
     Some(result)
 }
 
-pub fn read_cuesheet_block<B: ByteStream>(reader: &mut B, cues: &mut Vec<Cue>) -> Result<()> {
+pub fn read_cuesheet_block<B: ReadBytes>(reader: &mut B, cues: &mut Vec<Cue>) -> Result<()> {
     // Read cuesheet catalog number. The catalog number only allows printable ASCII characters.
     let mut catalog_number_buf = vec![0u8; 128];
     reader.read_buf_exact(&mut catalog_number_buf)?;
 
     let _catalog_number = match printable_ascii_to_string(&catalog_number_buf) {
         Some(s) => s,
-        None => return decode_error("cuesheet catalog number contains invalid characters"),
+        None => return decode_error("flac: cuesheet catalog number contains invalid characters"),
     };
 
     // Number of lead-in samples.
@@ -260,13 +260,13 @@ pub fn read_cuesheet_block<B: ByteStream>(reader: &mut B, cues: &mut Vec<Cue>) -
 
     // Lead-in should be non-zero only for CD-DA cuesheets.
     if !is_cdda && n_lead_in_samples > 0 {
-        return decode_error("cuesheet lead-in samples should be zero if not CD-DA");
+        return decode_error("flac: cuesheet lead-in samples should be zero if not CD-DA");
     }
 
     // Next 258 bytes (read as 129 u16's) must be zero.
     for _ in 0..129 {
         if reader.read_be_u16()? != 0 {
-            return decode_error("cuesheet reserved bits should be zero");
+            return decode_error("flac: cuesheet reserved bits should be zero");
         }
     }
 
@@ -274,12 +274,12 @@ pub fn read_cuesheet_block<B: ByteStream>(reader: &mut B, cues: &mut Vec<Cue>) -
 
     // There should be at-least one track in the cuesheet.
     if n_tracks == 0 {
-        return decode_error("cuesheet must have at-least one track");
+        return decode_error("flac: cuesheet must have at-least one track");
     }
 
     // CD-DA cuesheets must have no more than 100 tracks (99 audio tracks + lead-out track)
     if is_cdda && n_tracks > 100 {
-        return decode_error("cuesheets for CD-DA must not have more than 100 tracks");
+        return decode_error("flac: cuesheets for CD-DA must not have more than 100 tracks");
     }
 
     for _ in 0..n_tracks {
@@ -289,7 +289,7 @@ pub fn read_cuesheet_block<B: ByteStream>(reader: &mut B, cues: &mut Vec<Cue>) -
     Ok(())
 }
 
-fn read_cuesheet_track<B: ByteStream>(
+fn read_cuesheet_track<B: ReadBytes>(
     reader: &mut B,
     is_cdda: bool,
     cues: &mut Vec<Cue>
@@ -300,7 +300,7 @@ fn read_cuesheet_track<B: ByteStream>(
     // INDEX 01) on the CD. Therefore, the offset must be a multiple of 588 samples
     // (588 samples = 44100 samples/sec * 1/75th of a sec).
     if is_cdda && n_offset_samples % 588 != 0 {
-        return decode_error("cuesheet track sample offset is not a multiple of 588 for CD-DA");
+        return decode_error("flac: cuesheet track sample offset is not a multiple of 588 for CD-DA");
     }
 
     let number = u32::from(reader.read_u8()?);
@@ -308,13 +308,13 @@ fn read_cuesheet_track<B: ByteStream>(
     // A track number of 0 is disallowed in all cases. For CD-DA cuesheets, track 0 is reserved for
     // lead-in.
     if number == 0 {
-        return decode_error("cuesheet track number of 0 not allowed");
+        return decode_error("flac: cuesheet track number of 0 not allowed");
     }
 
     // For CD-DA cuesheets, only track numbers 1-99 are allowed for regular tracks and 170 for
     // lead-out.
     if is_cdda && number > 99 && number != 170 {
-        return decode_error("cuesheet track numbers greater than 99 are not allowed for CD-DA");
+        return decode_error("flac: cuesheet track numbers greater than 99 are not allowed for CD-DA");
     }
 
     let mut isrc_buf = vec![0u8; 12];
@@ -322,7 +322,7 @@ fn read_cuesheet_track<B: ByteStream>(
 
     let isrc = match printable_ascii_to_string(&isrc_buf) {
         Some(s) => s,
-        None => return decode_error("cuesheet track ISRC contains invalid characters"),
+        None => return decode_error("flac: cuesheet track ISRC contains invalid characters"),
     };
 
     // Next 14 bytes are reserved. However, the first two bits are flags. Consume the reserved bytes
@@ -334,13 +334,13 @@ fn read_cuesheet_track<B: ByteStream>(
     let _use_pre_emphasis = (flags & 0x4000) == 0x4000;
 
     if flags & 0x3fff != 0 {
-        return decode_error("cuesheet track reserved bits should be zero");
+        return decode_error("flac: cuesheet track reserved bits should be zero");
     }
 
     // Consume the remaining 12 bytes read in 3 u32 chunks.
     for _ in 0..3 {
         if reader.read_be_u32()? != 0 {
-            return decode_error("cuesheet track reserved bits should be zero");
+            return decode_error("flac: cuesheet track reserved bits should be zero");
         }
     }
 
@@ -348,7 +348,7 @@ fn read_cuesheet_track<B: ByteStream>(
 
     // For CD-DA cuesheets, the track index cannot exceed 100 indicies.
     if is_cdda && n_indicies > 100 {
-        return decode_error("cuesheet track indicies cannot exceed 100 for CD-DA");
+        return decode_error("flac: cuesheet track indicies cannot exceed 100 for CD-DA");
     }
 
     let mut cue = Cue {
@@ -370,7 +370,7 @@ fn read_cuesheet_track<B: ByteStream>(
     Ok(())
 }
 
-fn read_cuesheet_track_index<B: ByteStream>(reader: &mut B, is_cdda: bool) -> Result<CuePoint> {
+fn read_cuesheet_track_index<B: ReadBytes>(reader: &mut B, is_cdda: bool) -> Result<CuePoint> {
     let n_offset_samples = reader.read_be_u64()?;
     let idx_point_enc = reader.read_be_u32()?;
 
@@ -378,12 +378,12 @@ fn read_cuesheet_track_index<B: ByteStream>(reader: &mut B, is_cdda: bool) -> Re
     // (588 samples = 44100 samples/sec * 1/75th of a sec).
     if is_cdda && n_offset_samples % 588 != 0 {
         return decode_error(
-            "cuesheet track index point sample offset is not a multiple of 588 for CD-DA"
+            "flac: cuesheet track index point sample offset is not a multiple of 588 for CD-DA"
         );
     }
 
     if idx_point_enc & 0x00ff_ffff != 0 {
-        return decode_error("cuesheet track index reserved bits should be 0");
+        return decode_error("flac: cuesheet track index reserved bits should be 0");
     }
 
     // TODO: Should be 0 or 1 for the first index for CD-DA.
@@ -395,7 +395,7 @@ fn read_cuesheet_track_index<B: ByteStream>(reader: &mut B, is_cdda: bool) -> Re
     })
 }
 
-pub fn read_application_block<B : ByteStream>(
+pub fn read_application_block<B : ReadBytes>(
     reader: &mut B,
     block_length: u32,
 ) -> Result<VendorData> {
@@ -415,7 +415,7 @@ pub fn read_application_block<B : ByteStream>(
     Ok(VendorData { ident, data })
 }
 
-pub fn read_picture_block<B : ByteStream>(
+pub fn read_picture_block<B : ReadBytes>(
     reader: &mut B,
     metadata: &mut MetadataBuilder,
 ) -> Result<()> {
@@ -431,7 +431,7 @@ pub fn read_picture_block<B : ByteStream>(
     // Convert Media Type bytes to an ASCII string. Non-printable ASCII characters are invalid.
     let media_type = match printable_ascii_to_string(&media_type_buf) {
         Some(s) => s,
-        None => return decode_error("picture mime-type contains invalid characters"),
+        None => return decode_error("flac: picture mime-type contains invalid characters"),
     };
 
     // Read the description length in bytes.
@@ -444,8 +444,7 @@ pub fn read_picture_block<B : ByteStream>(
     let desc = String::from_utf8_lossy(&desc_buf);
 
     // Convert description bytes into a standard Vorbis DESCRIPTION tag.
-    let mut tags = Vec::<Tag>::new();
-    tags.push(Tag::new(Some(StandardTagKey::Description), "DESCRIPTION", Value::from(desc)));
+    let tags = vec![Tag::new(Some(StandardTagKey::Description), "DESCRIPTION", Value::from(desc))];
 
     // Read the width, and height of the visual.
     let width = reader.read_be_u32()?;
@@ -495,7 +494,7 @@ pub struct MetadataBlockHeader {
 }
 
 impl MetadataBlockHeader {
-    pub fn read<B : ByteStream>(reader: &mut B) -> Result<MetadataBlockHeader> {
+    pub fn read<B : ReadBytes>(reader: &mut B) -> Result<MetadataBlockHeader> {
         let header_enc = reader.read_u8()?;
 
         // First bit of the header indicates if this is the last metadata block.
